@@ -1,6 +1,6 @@
 /*
  * Linux usbfs backend for libusb
- * Copyright (C) 2007-2008 Daniel Drake <dsd@gentoo.org>
+ * Copyright (C) 2007-2009 Daniel Drake <dsd@gentoo.org>
  * Copyright (c) 2001 Johannes Erdfelt <johannes@erdfelt.com>
  *
  * This library is free software; you can redistribute it and/or
@@ -31,6 +31,7 @@
 #include <sys/ioctl.h>
 #include <sys/stat.h>
 #include <sys/types.h>
+#include <sys/utsname.h>
 #include <unistd.h>
 
 #include "libusb.h"
@@ -71,6 +72,20 @@
 
 static const char *usbfs_path = NULL;
 
+/* Linux 2.6.32 adds support for a bulk continuation URB flag. this should
+ * be set on all URBs in the transfer except the first. also set the
+ * SHORT_NOT_OK flag on all of them, to raise error conditions on short
+ * transfers.
+ * then, on any error except a cancellation, all URBs until the next
+ * non-continuation URB will be cancelled with the endpoint disabled,
+ * meaning that no more data can creep in during the time it takes us to
+ * cancel the remaining URBs. */
+static int supports_flag_bulk_continuation = -1;
+
+/* clock ID for monotonic clock, as not all clock sources are available on all
+ * systems. appropriate choice made at initialization time. */
+static clockid_t monotonic_clkid = -1;
+
 /* do we have a busnum to relate devices? this also implies that we can read
  * the active configuration through bConfigurationValue */
 static int sysfs_can_relate_devices = -1;
@@ -172,6 +187,55 @@ static const char *find_usbfs_path(void)
 	return ret;
 }
 
+static clockid_t find_monotonic_clock(void)
+{
+	struct timespec ts;
+	int i;
+	const clockid_t clktypes[] = {
+		/* the most monotonic clock I know about, but only available since
+		 * Linux 2.6.28, and not even available in glibc-2.10 */
+#ifndef CLOCK_MONOTONIC_RAW
+#define CLOCK_MONOTONIC_RAW 4
+#endif
+		CLOCK_MONOTONIC_RAW,
+
+		/* a monotonic clock, but it's not available on all architectures,
+		 * and is susceptible to ntp adjustments */
+		CLOCK_MONOTONIC,
+
+		/* the fallback option */
+		CLOCK_REALTIME,
+	};
+
+	for (i = 0; i < (sizeof(clktypes) / sizeof(*clktypes)); i++) {
+		int r = clock_gettime(clktypes[i], &ts);
+		if (r == 0) {
+			usbi_dbg("clock %d selected", clktypes[i]);
+			return r;
+		}
+		usbi_dbg("clock %d doesn't work", clktypes[i]);
+	}
+
+	return -1;
+}
+
+/* bulk continuation URB flag available from Linux 2.6.32 */
+static int check_flag_bulk_continuation(void)
+{
+	struct utsname uts;
+	int sublevel;
+
+	if (uname(&uts) < 0)
+		return -1;
+	if (strlen(uts.release) < 4)
+		return 0;
+	if (strncmp(uts.release, "2.6.", 4) != 0)
+		return 0;
+
+	sublevel = atoi(uts.release + 4);
+	return sublevel >= 32;
+}
+
 static int op_init(struct libusb_context *ctx)
 {
 	struct stat statbuf;
@@ -183,6 +247,25 @@ static int op_init(struct libusb_context *ctx)
 		return LIBUSB_ERROR_OTHER;
 	}
 
+	if (monotonic_clkid == -1) {
+		monotonic_clkid = find_monotonic_clock();
+		if (monotonic_clkid == -1) {
+			usbi_err(ctx, "could not find working monotonic clock");
+			return LIBUSB_ERROR_OTHER;
+		}
+	}
+
+	if (supports_flag_bulk_continuation == -1) {
+		supports_flag_bulk_continuation = check_flag_bulk_continuation();
+		if (supports_flag_bulk_continuation == -1) {
+			usbi_err(ctx, "error checking for bulk continuation support");
+			return LIBUSB_ERROR_OTHER;
+		}
+	}
+
+	if (supports_flag_bulk_continuation)
+		usbi_dbg("bulk continuation flag supported");
+
 	r = stat(SYSFS_DEVICE_PATH, &statbuf);
 	if (r == 0 && S_ISDIR(statbuf.st_mode)) {
 		usbi_dbg("found usb devices in sysfs");
@@ -1276,7 +1359,9 @@ static int submit_bulk_transfer(struct usbi_transfer *itransfer,
 	int num_urbs = transfer->length / MAX_BULK_BUFFER_LENGTH;
 	int last_urb_partial = 0;
 
-	if ((transfer->length % MAX_BULK_BUFFER_LENGTH) > 0) {
+	if (transfer->length == 0) {
+		num_urbs = 1;
+	} else if ((transfer->length % MAX_BULK_BUFFER_LENGTH) > 0) {
 		last_urb_partial = 1;
 		num_urbs++;
 	}
@@ -1298,11 +1383,18 @@ static int submit_bulk_transfer(struct usbi_transfer *itransfer,
 		urb->type = urb_type;
 		urb->endpoint = transfer->endpoint;
 		urb->buffer = transfer->buffer + (i * MAX_BULK_BUFFER_LENGTH);
+		if (supports_flag_bulk_continuation)
+			urb->flags = USBFS_URB_SHORT_NOT_OK;
 		if (i == num_urbs - 1 && last_urb_partial)
 			urb->buffer_length = transfer->length % MAX_BULK_BUFFER_LENGTH;
+		else if (transfer->length == 0)
+			urb->buffer_length = 0;
 		else
 			urb->buffer_length = MAX_BULK_BUFFER_LENGTH;
 
+		if (i > 0 && supports_flag_bulk_continuation)
+			urb->flags |= USBFS_URB_BULK_CONTINUATION;
+
 		r = ioctl(dpriv->fd, IOCTL_USBFS_SUBMITURB, urb);
 		if (r < 0) {
 			int j;
@@ -1701,63 +1793,91 @@ static int handle_bulk_completion(struct usbi_transfer *itransfer,
 	int urb_idx = urb - tpriv->urbs;
 	enum libusb_transfer_status status = LIBUSB_TRANSFER_COMPLETED;
 
+	pthread_mutex_lock(&itransfer->lock);
 	usbi_dbg("handling completion status %d of bulk urb %d/%d", urb->status,
 		urb_idx + 1, num_urbs);
 
 	tpriv->num_retired++;
 
-	if (urb->status == 0 ||
-			(urb->status == -EOVERFLOW && urb->actual_length > 0))
-		itransfer->transferred += urb->actual_length;
-
 	if (tpriv->reap_action != NORMAL) {
 		/* cancelled, submit_fail, or completed early */
-		if (urb->status == 0 && tpriv->reap_action == COMPLETED_EARLY) {
-			/* FIXME we could solve this extreme corner case with a memmove
-			 * or something */
-			usbi_warn(ITRANSFER_CTX(itransfer), "SOME DATA LOST! "
-				"(completed early but remaining urb completed)");
+		usbi_dbg("abnormal reap: urb status %d", urb->status);
+
+		/* even though we're in the process of cancelling, it's possible that
+		 * we may receive some data in these URBs that we don't want to lose.
+		 * examples:
+		 * 1. while the kernel is cancelling all the packets that make up an
+		 *    URB, a few of them might complete. so we get back a successful
+		 *    cancellation *and* some data.
+		 * 2. we receive a short URB which marks the early completion condition,
+		 *    so we start cancelling the remaining URBs. however, we're too
+		 *    slow and another URB completes (or at least completes partially).
+		 *
+		 * When this happens, our objectives are not to lose any "surplus" data,
+		 * and also to stick it at the end of the previously-received data
+		 * (closing any holes), so that libusb reports the total amount of
+		 * transferred data and presents it in a contiguous chunk.
+		 */
+		if (urb->actual_length > 0) {
+			struct libusb_transfer *transfer =
+				__USBI_TRANSFER_TO_LIBUSB_TRANSFER(itransfer);
+			unsigned char *target = transfer->buffer + itransfer->transferred;
+			usbi_dbg("received %d bytes of surplus data", urb->actual_length);
+			if (urb->buffer != target) {
+				usbi_dbg("moving surplus data from offset %d to offset %d",
+					(unsigned char *) urb->buffer - transfer->buffer,
+					target - transfer->buffer);
+				memmove(target, urb->buffer, urb->actual_length);
+			}
+			itransfer->transferred += urb->actual_length;
 		}
-		usbi_dbg("CANCEL: urb status %d", urb->status);
 
 		if (tpriv->num_retired == num_urbs) {
-			usbi_dbg("CANCEL: last URB handled, reporting");
+			usbi_dbg("abnormal reap: last URB handled, reporting");
 			if (tpriv->reap_action == CANCELLED) {
 				free(tpriv->urbs);
 				tpriv->urbs = NULL;
+				pthread_mutex_unlock(&itransfer->lock);
 				usbi_handle_transfer_cancellation(itransfer);
-				return 0;
+				goto out_unlock;
 			}
 			if (tpriv->reap_action != COMPLETED_EARLY)
 				status = LIBUSB_TRANSFER_ERROR;
-			goto out;
+			goto completed;
 		}
-		return 0;
+		goto out_unlock;
 	}
 
+	if (urb->status == 0 || urb->status == -EREMOTEIO ||
+			(urb->status == -EOVERFLOW && urb->actual_length > 0))
+		itransfer->transferred += urb->actual_length;
+
+
 	switch (urb->status) {
 	case 0:
 		break;
+	case -EREMOTEIO: /* short transfer */
+		break;
 	case -EPIPE:
 		usbi_dbg("detected endpoint stall");
 		status = LIBUSB_TRANSFER_STALL;
-		goto out;
+		goto completed;
 	case -EOVERFLOW:
 		/* overflow can only ever occur in the last urb */
 		usbi_dbg("overflow, actual_length=%d", urb->actual_length);
 		status = LIBUSB_TRANSFER_OVERFLOW;
-		goto out;
+		goto completed;
 	case -ETIME:
 	case -EPROTO:
 	case -EILSEQ:
 		usbi_dbg("low level error %d", urb->status);
 		status = LIBUSB_TRANSFER_ERROR;
-		goto out;
+		goto completed;
 	default:
 		usbi_warn(ITRANSFER_CTX(itransfer),
 			"unrecognised urb status %d", urb->status);
 		status = LIBUSB_TRANSFER_ERROR;
-		goto out;
+		goto completed;
 	}
 
 	/* if we're the last urb or we got less data than requested then we're
@@ -1778,21 +1898,29 @@ static int handle_bulk_completion(struct usbi_transfer *itransfer,
 		 * before reporting results */
 		tpriv->reap_action = COMPLETED_EARLY;
 		for (i = urb_idx + 1; i < tpriv->num_urbs; i++) {
+			/* remaining URBs with continuation flag are automatically
+			 * cancelled by the kernel */
+			if (tpriv->urbs[i].flags & USBFS_URB_BULK_CONTINUATION)
+				continue;
 			int r = ioctl(dpriv->fd, IOCTL_USBFS_DISCARDURB, &tpriv->urbs[i]);
 			if (r && errno != EINVAL)
 				usbi_warn(TRANSFER_CTX(transfer),
 					"unrecognised discard errno %d", errno);
 		}
-		return 0;
+		goto out_unlock;
 	} else {
-		return 0;
+		goto out_unlock;
 	}
 
-out:
+completed:
 	free(tpriv->urbs);
 	tpriv->urbs = NULL;
+	pthread_mutex_unlock(&itransfer->lock);
 	usbi_handle_transfer_completion(itransfer, status);
 	return 0;
+out_unlock:
+	pthread_mutex_unlock(&itransfer->lock);
+	return 0;
 }
 
 static int handle_iso_completion(struct usbi_transfer *itransfer,
@@ -1805,6 +1933,7 @@ static int handle_iso_completion(struct usbi_transfer *itransfer,
 	int urb_idx = 0;
 	int i;
 
+	pthread_mutex_lock(&itransfer->lock);
 	for (i = 0; i < num_urbs; i++) {
 		if (urb == tpriv->iso_urbs[i]) {
 			urb_idx = i + 1;
@@ -1813,6 +1942,7 @@ static int handle_iso_completion(struct usbi_transfer *itransfer,
 	}
 	if (urb_idx == 0) {
 		usbi_err(TRANSFER_CTX(transfer), "could not locate urb!");
+		pthread_mutex_unlock(&itransfer->lock);
 		return LIBUSB_ERROR_NOT_FOUND;
 	}
 
@@ -1839,13 +1969,17 @@ static int handle_iso_completion(struct usbi_transfer *itransfer,
 		if (tpriv->num_retired == num_urbs) {
 			usbi_dbg("CANCEL: last URB handled, reporting");
 			free_iso_urbs(tpriv);
-			if (tpriv->reap_action == CANCELLED)
+			if (tpriv->reap_action == CANCELLED) {
+				pthread_mutex_unlock(&itransfer->lock);
 				usbi_handle_transfer_cancellation(itransfer);
-			else
+			} else {
+				pthread_mutex_unlock(&itransfer->lock);
 				usbi_handle_transfer_completion(itransfer,
 					LIBUSB_TRANSFER_ERROR);
+			}
+			return 0;
 		}
-		return 0;
+		goto out;
 	}
 
 	switch (urb->status) {
@@ -1867,9 +2001,13 @@ static int handle_iso_completion(struct usbi_transfer *itransfer,
 	if (urb_idx == num_urbs) {
 		usbi_dbg("last URB in transfer --> complete!");
 		free_iso_urbs(tpriv);
+		pthread_mutex_unlock(&itransfer->lock);
 		usbi_handle_transfer_completion(itransfer, LIBUSB_TRANSFER_COMPLETED);
+		return 0;
 	}
 
+out:
+	pthread_mutex_unlock(&itransfer->lock);
 	return 0;
 }
 
@@ -1879,6 +2017,7 @@ static int handle_control_completion(struct usbi_transfer *itransfer,
 	struct linux_transfer_priv *tpriv = usbi_transfer_get_os_priv(itransfer);
 	int status;
 
+	pthread_mutex_lock(&itransfer->lock);
 	usbi_dbg("handling completion status %d", urb->status);
 
 	if (urb->status == 0)
@@ -1890,6 +2029,7 @@ static int handle_control_completion(struct usbi_transfer *itransfer,
 				"cancel: unrecognised urb status %d", urb->status);
 		free(tpriv->urbs);
 		tpriv->urbs = NULL;
+		pthread_mutex_unlock(&itransfer->lock);
 		usbi_handle_transfer_cancellation(itransfer);
 		return 0;
 	}
@@ -1918,6 +2058,7 @@ static int handle_control_completion(struct usbi_transfer *itransfer,
 
 	free(tpriv->urbs);
 	tpriv->urbs = NULL;
+	pthread_mutex_unlock(&itransfer->lock);
 	usbi_handle_transfer_completion(itransfer, status);
 	return 0;
 }
@@ -2008,7 +2149,7 @@ static int op_clock_gettime(int clk_id, struct timespec *tp)
 {
 	switch (clk_id) {
 	case USBI_CLOCK_MONOTONIC:
-		return clock_gettime(CLOCK_MONOTONIC, tp);
+		return clock_gettime(monotonic_clkid, tp);
 	case USBI_CLOCK_REALTIME:
 		return clock_gettime(CLOCK_REALTIME, tp);
 	default:
